@@ -24,16 +24,17 @@ import (
 )
 
 type Database struct {
-	BuildArgs  []dc.BuildArg
-	RunOptions *dt.RunOptions
-	Exec       []string
-	Driver     string
-	URL        string
-	DockerPort string
-	Resource   *dt.Resource
-	DB         *sql.DB
-	Opts       []metadata.ReaderOption
-	Reader     metadata.BasicReader
+	BuildArgs    []dc.BuildArg
+	RunOptions   *dt.RunOptions
+	Exec         []string
+	Driver       string
+	URL          string
+	ReadinessURL string
+	DockerPort   string
+	Resource     *dt.Resource
+	DB           *sql.DB
+	Opts         []metadata.ReaderOption
+	Reader       metadata.BasicReader
 }
 
 const (
@@ -85,6 +86,7 @@ var (
 				infos.WithPlaceholder(func(int) string { return "?" }),
 				infos.WithCheckConstraints(false),
 				infos.WithCustomClauses(map[infos.ClauseName]string{
+					infos.ColumnsDataType:                 "column_type",
 					infos.ColumnsNumericPrecRadix:         "10",
 					infos.FunctionColumnsNumericPrecRadix: "10",
 					infos.ConstraintIsDeferrable:          "''",
@@ -104,10 +106,11 @@ var (
 				Name: "usql-sqlserver",
 				Env:  []string{"ACCEPT_EULA=Y", "SA_PASSWORD=" + pw},
 			},
-			Exec:       []string{"/opt/mssql-tools/bin/sqlcmd", "-S", "localhost", "-U", "sa", "-P", pw, "-d", "master", "-i", "/schema/sql-server-sakila-schema.sql"},
-			Driver:     "sqlserver",
-			URL:        "sqlserver://sa:" + url.QueryEscape(pw) + "@127.0.0.1:%s?database=sakila",
-			DockerPort: "1433/tcp",
+			Exec:         []string{"/opt/mssql-tools/bin/sqlcmd", "-S", "localhost", "-U", "sa", "-P", pw, "-d", "master", "-i", "/schema/sql-server-sakila-schema.sql"},
+			Driver:       "sqlserver",
+			URL:          "sqlserver://sa:" + url.QueryEscape(pw) + "@127.0.0.1:%s?database=sakila",
+			ReadinessURL: "sqlserver://sa:" + url.QueryEscape(pw) + "@127.0.0.1:%s",
+			DockerPort:   "1433/tcp",
 			Opts: []metadata.ReaderOption{
 				infos.WithPlaceholder(func(n int) string { return fmt.Sprintf("@p%d", n) }),
 				infos.WithIndexes(false),
@@ -184,7 +187,7 @@ func TestMain(m *testing.M) {
 		log.Fatalf("Could not connect to docker: %s", err)
 	}
 
-	for _, db := range dbs {
+	for dbName, db := range dbs {
 		var ok bool
 		db.Resource, ok = pool.ContainerByName(db.RunOptions.Name)
 		if !ok {
@@ -198,19 +201,14 @@ func TestMain(m *testing.M) {
 			}
 		}
 
-		// exponential backoff-retry, because the application in the container might not be ready to accept connections yet
-		if err := pool.Retry(func() error {
-			hostPort := db.Resource.GetPort(db.DockerPort)
-			var err error
-			db.DB, err = sql.Open(db.Driver, fmt.Sprintf(db.URL, hostPort))
-			if err != nil {
-				return err
-			}
-			return db.DB.Ping()
-		}); err != nil {
-			log.Fatal("Timed out waiting for db: ", err)
+		url := db.URL
+		if db.ReadinessURL != "" {
+			url = db.ReadinessURL
+		}
+		port := db.Resource.GetPort(db.DockerPort)
+		if db.DB, err = waitForDbConnection(db.Driver, pool, url, port); err != nil {
+			log.Fatalf("Timed out waiting for %s: %s", dbName, err)
 		}
-		db.Reader = infos.New(db.Opts...)(db.DB).(metadata.BasicReader)
 
 		if len(db.Exec) != 0 {
 			exitCode, err := db.Resource.Exec(db.Exec, dt.ExecOptions{
@@ -223,6 +221,14 @@ func TestMain(m *testing.M) {
 				log.Fatal("Could not load schema: ", err)
 			}
 		}
+
+		// Reconnect with actual URL if a separate URL for readiness checking was used
+		if db.ReadinessURL != "" {
+			if db.DB, err = waitForDbConnection(db.Driver, pool, db.URL, port); err != nil {
+				log.Fatalf("Timed out waiting for %s: %s", dbName, err)
+			}
+		}
+		db.Reader = infos.New(db.Opts...)(db.DB).(metadata.BasicReader)
 	}
 
 	code := m.Run()
@@ -239,6 +245,22 @@ func TestMain(m *testing.M) {
 	os.Exit(code)
 }
 
+func waitForDbConnection(driver string, pool *dt.Pool, url string, port string) (*sql.DB, error) {
+	// exponential backoff-retry, because the application in the container might not be ready to accept connections yet
+	var db *sql.DB
+	if err := pool.Retry(func() error {
+		var err error
+		db, err = sql.Open(driver, fmt.Sprintf(url, port))
+		if err != nil {
+			return err
+		}
+		return db.Ping()
+	}); err != nil {
+		return nil, err
+	}
+	return db, nil
+}
+
 func TestSchemas(t *testing.T) {
 	expected := map[string]string{
 		"pgsql":     "information_schema, pg_catalog, pg_toast, public",
@@ -310,12 +332,15 @@ func TestColumns(t *testing.T) {
 		"sqlserver": "film%",
 		"trino":     "orders",
 	}
-	expected := map[string]string{
+	expectedColumns := map[string]string{
 		"pgsql":     "film_id, title, description, release_year, language_id, original_language_id, rental_duration, rental_rate, length, replacement_cost, rating, last_update, special_features, fulltext, actor_id, film_id, last_update, film_id, category_id, last_update, fid, title, description, category, price, length, rating, actors",
 		"mysql":     "film_id, title, description, release_year, language_id, original_language_id, rental_duration, rental_rate, length, replacement_cost, rating, special_features, last_update, actor_id, film_id, last_update, film_id, category_id, last_update, FID, title, description, category, price, length, rating, actors, film_id, title, description",
 		"sqlserver": "film_id, title, description, release_year, language_id, original_language_id, rental_duration, rental_rate, length, replacement_cost, rating, special_features, last_update, actor_id, film_id, last_update, film_id, category_id, last_update, FID, title, description, category, price, length, rating, actors, film_id, title, description",
 		"trino":     "orderkey, custkey, orderstatus, totalprice, orderdate, orderpriority, clerk, shippriority, comment",
 	}
+	expectedTypes := map[string]string{
+		"mysql": "int unsigned, varchar(255), text, year, int unsigned, int unsigned, tinyint unsigned, decimal(4,2), smallint unsigned, decimal(5,2), enum('G','PG','PG-13','R','NC-17'), set('Trailers','Commentaries','Deleted Scenes','Behind the Scenes'), timestamp, int unsigned, int unsigned, timestamp, int unsigned, int unsigned, timestamp, int unsigned, varchar(255), text, varchar(25), decimal(4,2), smallint unsigned, enum('G','PG','PG-13','R','NC-17'), text, int, varchar(255), text",
+	}
 	for dbName, db := range dbs {
 		r := db.Reader
 
@@ -325,12 +350,18 @@ func TestColumns(t *testing.T) {
 		}
 
 		names := []string{}
+		types := []string{}
 		for result.Next() {
 			names = append(names, result.Get().Name)
+			types = append(types, result.Get().DataType)
 		}
-		actual := strings.Join(names, ", ")
-		if actual != expected[dbName] {
-			t.Errorf("Wrong %s column names, expected:\n  %v, got:\n  %v", dbName, expected[dbName], names)
+		actualColumns := strings.Join(names, ", ")
+		actualTypes := strings.Join(types, ", ")
+		if expected, ok := expectedColumns[dbName]; ok && actualColumns != expected {
+			t.Errorf("Wrong %s column names, expected:\n  %v, got:\n  %v", dbName, expected, names)
+		}
+		if expected, ok := expectedTypes[dbName]; ok && actualTypes != expected {
+			t.Errorf("Wrong %s column types, expected:\n  %v, got:\n  %v", dbName, expected, types)
 		}
 	}
 }
