@@ -1,12 +1,8 @@
 package metadata
 
 import (
-	"errors"
-)
-
-var (
-	ErrNotSupported  = errors.New("error: not supported")
-	ErrScanArgsCount = errors.New("error: wrong number of arguments for Scan()")
+	"github.com/xo/dburl"
+	"github.com/xo/usql/text"
 )
 
 // ExtendedReader of all database metadata in a structured format.
@@ -15,6 +11,7 @@ type ExtendedReader interface {
 	SchemaReader
 	TableReader
 	ColumnReader
+	ColumnStatReader
 	IndexReader
 	IndexColumnReader
 	TriggerReader
@@ -56,6 +53,12 @@ type ColumnReader interface {
 	Columns(Filter) (*ColumnSet, error)
 }
 
+// ColumnStatsReader lists table column statistics.
+type ColumnStatReader interface {
+	Reader
+	ColumnStats(Filter) (*ColumnStatSet, error)
+}
+
 // IndexReader lists table indexes.
 type IndexReader interface {
 	Reader
@@ -133,17 +136,19 @@ type Filter struct {
 // Writer of database metadata in a human readable format.
 type Writer interface {
 	// DescribeFunctions \df, \dfa, \dfn, \dft, \dfw, etc.
-	DescribeFunctions(string, string, bool, bool) error
+	DescribeFunctions(*dburl.URL, string, string, bool, bool) error
 	// DescribeTableDetails \d foo
-	DescribeTableDetails(string, bool, bool) error
+	DescribeTableDetails(*dburl.URL, string, bool, bool) error
 	// ListAllDbs \l
-	ListAllDbs(string, bool) error
+	ListAllDbs(*dburl.URL, string, bool) error
 	// ListTables \dt, \dv, \dm, etc.
-	ListTables(string, string, bool, bool) error
+	ListTables(*dburl.URL, string, string, bool, bool) error
 	// ListSchemas \dn
-	ListSchemas(string, bool, bool) error
+	ListSchemas(*dburl.URL, string, bool, bool) error
 	// ListIndexes \di
-	ListIndexes(string, bool, bool) error
+	ListIndexes(*dburl.URL, string, bool, bool) error
+	// ShowStats \ss
+	ShowStats(*dburl.URL, string, string, bool, int) error
 }
 
 type CatalogSet struct {
@@ -224,6 +229,7 @@ func NewTableSet(v []Table) *TableSet {
 				"Name",
 				"Type",
 
+				"Rows",
 				"Size",
 				"Comment",
 			},
@@ -240,6 +246,7 @@ type Table struct {
 	Schema  string
 	Name    string
 	Type    string
+	Rows    int64
 	Size    string
 	Comment string
 }
@@ -250,6 +257,7 @@ func (t Table) values() []interface{} {
 		t.Schema,
 		t.Name,
 		t.Type,
+		t.Rows,
 		t.Size,
 		t.Comment,
 	}
@@ -330,6 +338,73 @@ func (c Column) values() []interface{} {
 	}
 }
 
+type ColumnStatSet struct {
+	resultSet
+}
+
+func NewColumnStatSet(v []ColumnStat) *ColumnStatSet {
+	r := make([]Result, len(v))
+	for i := range v {
+		r[i] = &v[i]
+	}
+	return &ColumnStatSet{
+		resultSet: resultSet{
+			results: r,
+			columns: []string{
+				"Catalog",
+				"Schema",
+				"Table",
+				"Name",
+
+				"Average width",
+				"Nulls fraction",
+				"Distinct values",
+				"Minimum value",
+				"Maximum value",
+				"Mean value",
+				"Top N common values",
+				"Top N values freqs",
+			},
+		},
+	}
+}
+
+func (c ColumnStatSet) Get() *ColumnStat {
+	return c.results[c.current-1].(*ColumnStat)
+}
+
+type ColumnStat struct {
+	Catalog     string
+	Schema      string
+	Table       string
+	Name        string
+	AvgWidth    int
+	NullFrac    float64
+	NumDistinct int64
+	Min         string
+	Max         string
+	Mean        string
+	TopN        []string
+	TopNFreqs   []float64
+}
+
+func (c ColumnStat) values() []interface{} {
+	return []interface{}{
+		c.Catalog,
+		c.Schema,
+		c.Table,
+		c.Name,
+		c.AvgWidth,
+		c.NullFrac,
+		c.NumDistinct,
+		c.Min,
+		c.Max,
+		c.Mean,
+		c.TopN,
+		c.TopNFreqs,
+	}
+}
+
 type IndexSet struct {
 	resultSet
 }
@@ -819,7 +894,7 @@ func (r resultSet) Scan(dest ...interface{}) error {
 		v = r.scanValues(r.results[r.current-1])
 	}
 	if len(v) != len(dest) {
-		return ErrScanArgsCount
+		return text.ErrWrongNumberOfArguments
 	}
 	for i, d := range dest {
 		p := d.(*interface{})
